module Gargantext.Components.GraphQL.Tree where

import Data.Maybe (Maybe(..))
import Gargantext.Components.GraphQL.Node (Node, NodeQ, nodeQ, getNodeByIdQuery, getNodesQuery)
import Gargantext.Prelude
import Gargantext.Routes (AppRoute)
import Gargantext.Types (NodeType)
import Gargantext.Utils.GraphQL as GGQL
import GraphQL.Client.Alias ((:), Alias)
import GraphQL.Client.Args ((=>>), Args, OrArg(ArgR), IgnoreArg)
import GraphQL.Client.Variable (Var(..))
import Type.Proxy (Proxy(..))

type TreeFirstLevel =
  { root :: Node
  , children :: Array Node
  , parent :: Maybe Node
  }

type BreadcrumbInfo =
  { parents :: Array Node }

-- | This is a composite query that is composed of calling "node" and
-- | "nodes" with appropriate arguments.
getTreeByRootIdQuery
  :: Int
  -> { root ::
         Alias (Proxy "node")
           ( Args
               { id :: Int }
               NodeQ
           )
     , children ::
         Alias (Proxy "nodes")
           ( Args
               { parentId :: Int }
               NodeQ
           )
     , parents ::
         Alias (Proxy "nodes")
           ( Args
               { containsChildId :: Int }
               NodeQ
           )
     }
getTreeByRootIdQuery id =
  { root:
      node : { id: id } =>> nodeQ
  , children:
      nodes : { parentId: id } =>> nodeQ
  , parents:
      nodes : { containsChildId: id } =>> nodeQ
  }
  where
  node = Proxy :: Proxy "node"
  nodes = Proxy :: Proxy "nodes"

breadcrumbQuery
  :: Int
  -> { parents ::
         Alias (Proxy "nodes")
           ( Args
               { deepChildId :: Int }
               NodeQ
           )
     }
breadcrumbQuery id =
  { parents:
      nodes : { deepChildId: id } =>>
        nodeQ
  }
  where
  nodes = Proxy :: Proxy "nodes"
